<?php 

class ofdfermaCheckModel extends waModel{

    private $_debug = 0;
    private $_config = null;
    
    protected $table = 'ofdferma_check';
    
    public function __construct($type = null, $writable = false)
    {
        parent::__construct($type, $writable);
        
        $this -> _initConfig();
    }

	private function _initConfig(){
        $this -> _config = wa('ofdferma')->getConfig();
    }
	
	private function updateConfig($data = array()){
		if ($data){
			$settings = $this -> _config -> getOption(null);
            
			$settings = array_merge($settings, $data);
			
			$config_file = $this -> _config -> getConfigPath('config.php');
			waUtils::varExportToFile($settings, $config_file);
			
			$this -> _config -> init();
		}
	}
	
	public function install(){
		
		$install = $this -> _config -> getOption('setting_install');
		if ( ! $install){
			$model = new waModel();
			$table = 'shop_product';

			try {
				$sql = 'SELECT ofdferma_nds FROM '.$table.' WHERE 0';
				$model->query($sql);
			} catch (waDbException $ex) {
				$sql = 'ALTER TABLE '.$table.' ADD ofdferma_nds VARCHAR(20) NOT NULL';
				$model->query($sql);
			}
			
			$this -> updateConfig(array(
				'setting_install' => 1
			));
		}
	}
    
    private function checkExists($order_id, $type) {
        return $this->query("SELECT COUNT(*) FROM {$this->table} WHERE `order_id` = '".(int)$order_id."' AND `type` = '".addslashes($type)."'")->fetchField();
    }

    public function setDebug($val) {
        $this->_debug = $val;
    }

    public function log_msg($mess, $status = false, $params = array()) {

        if (!$status && $this->_debug && $mess) {
            waLog::log((string)$mess, 'ofdferma.log'); 
            
            $ofd_email = $this -> _config -> getOption('setting_email');
            if ($ofd_email) {
                $message = new waMailMessage('OFD Ferma - Ошибка', $mess);
                $message->setTo($ofd_email);
                //$message->setFrom($from, $general['name']);

                $message->send();
            }
        }

        return array_merge(array(
            'status'    => $status,
            'mess'      => $mess
        ), $params);
    }

    public function getTextType($type) {
        $res = '';

        switch ($type) {
            case 'Income':
                $res = 'Приход';
                break;

            case 'IncomeReturn':
                $res = 'Возврат';
                break;
        }

        return $res;
    }

    private function formatFloat( $int, $count = 2){
        return	round( (float)$int , $count);
    }
    
    public function prepareData($order_data, $type) {

        $prefix     = 't-shop-script-3';
        
        $ofd_inn    = $this -> _config -> getOption('setting_inn');
        $ofd_nalog  = $this -> _config -> getOption('setting_nalog');
        $ofd_nds    = $this -> _config -> getOption('setting_nds');
        
        //свертка
        $ofd_collapse      = $this -> _config -> getOption('setting_collapse');
        $ofd_collapse_name = $this -> _config -> getOption('setting_collapse_name');
        
        
        // Get Order
        $id = $order_data['id'];
        $customer_phone = $order_data['contact']['phone']; 
        $customer_email = $order_data['contact']['email'];

        $products = $order_data['items'];

        if (!$products) {
            return $this->log_msg("В заказе #{$id} нет товаров");
        }

        //Формируем данные
        $data = array();
        $data['Request']['Inn'] = $ofd_inn;
        $data['Request']['Type'] = $type;
        $data['Request']['InvoiceId'] = $prefix . $id . '-' . $type;
        $data['Request']['LocalDate'] = date('Y-m-d\TH:i:s');
        $data['Request']['CustomerReceipt'] = array(
            'TaxationSystem' => $ofd_nalog,
            'Email' => $customer_email,
            'Phone' => $customer_phone,
            'Items' => array(),
        );

       
        // Get Items: Price / Sold / Email
        foreach ($products as $item) {
            $vat = '';
            
			/*if (isset($item['tax_percent'])){
				switch((int)$item['tax_percent']) {
					case 18 : $vat = 'Vat18'; break;
					case 10 : $vat = 'Vat10'; break;
					case 0  : $vat = 'Vat0'; break;
				}
			}*/
			$vat = $this -> getNdsProductById($item['product_id']);
				
            if ( ! in_array($vat, array('Vat0','Vat10','Vat18'))){
                $vat = $ofd_nds;
            }
            
            array_push($data['Request']['CustomerReceipt']['Items'], array(
                    'Label'     => $item['name'],
                    'Price'     => $this->formatFloat( $item['price'] ),
                    'Quantity'  => $this->formatFloat( $item['quantity'], 3),
                    'Amount'    => $this->formatFloat( $item['price'] * $item['quantity'] - $item['total_discount'] ),
                    'Vat'       => $vat,
                )
            );
        }

        if(isset($order_data['shipping']) && (float)$order_data['shipping'] > 0 ) {
            $price = $this->formatFloat( $order_data['shipping']); 
            $data['Request']['CustomerReceipt']['Items'][] = array(
                'Label'     => ( (isset($order_data['params']['shipping_name']) && !empty($order_data['params']['shipping_name'])) ? substr( $order_data['params']['shipping_name'], 0, 128) : 'Доставка'  ),
                'Price'     => $price,
                'Amount'    => $price,
                'Quantity'  => 1,
                'Vat'       => $ofd_nds 
            );
        }
        
        //Если включена свертка
        if ($ofd_collapse) {
            $pos_name = $ofd_collapse_name ? $ofd_collapse_name : 'Undefined';

            $sum = 0;
            foreach ($data['Request']['CustomerReceipt']['Items'] as $item) {
                $sum += $item['Amount'];
            }
            $data['Request']['CustomerReceipt']['Items'] = array(array(
                    'Label'     => $pos_name,
                    'Price'     => $sum,
                    'Quantity'  => 1,
                    'Amount'    => $sum,
                    'Vat'       => $ofd_nds,
            ));
        }

        return $data;
    }

	public function getNdsProductById($product_id){
		
		wa('shop');
		$model = new shopProductModel();
        $model->select('ofdferma_nds');
        
        $data = $model->getById((int)$product_id);
		
        return $data['ofdferma_nds'];
	}
	
    public function customErrorHandler($errno, $errstr, $errfile, $errline, array $errcontext) {
        $this->log_msg($errstr);

        if (0 === error_reporting()) {
            return false;
        }
        throw new ErrorException($errstr, 0, $errno, $errfile, $errline);
    }

    private function checkSettings() {
        $ofd_auth_url   = $this -> _config -> getOption('setting_auth_url');
        $ofd_api_url    = $this -> _config -> getOption('setting_api_url');
        $ofd_login      = $this -> _config -> getOption('setting_login');
        $ofd_pass       = $this -> _config -> getOption('setting_password');
        $ofd_nalog      = $this -> _config -> getOption('setting_nalog');
        $ofd_inn        = $this -> _config -> getOption('setting_inn');
        $ofd_nds        = $this -> _config -> getOption('setting_nds');

        return (
            $ofd_auth_url &&
            $ofd_api_url &&
            $ofd_login &&
            $ofd_pass &&
            $ofd_nalog &&
            $ofd_inn &&
            $ofd_nds
        );
    }

    private function checkToken() {
        $ofd_token          = $this -> _config -> getOption('setting_token');
        $ofd_token_exp_date = $this -> _config -> getOption('setting_token_exp_date');
        
        if ($ofd_token && ($ofd_token_exp_date > (time() - 10))) {
            return $ofd_token;
        } else {
            return false;
        }
    }

    private function getHTTPOpt($data) {
        $options = array(
            "ssl" => array(
                "verify_peer" => false,
                "verify_peer_name" => false,
            ),
            'http' => array(
                'timeout' => 10,
                'ignore_errors' => true,
                'content' => json_encode($data),
                'header' => "Content-type: application/json\r\n" .
                "Accept: application/json" . "\r\n",
                "Content-Length: " . strlen(json_encode($data)) . "\r\n",
                'method' => 'POST',
            )
        );

        return $options;
    }

    public function UpdateChecksStatus($ids = array()) {
        $sWhere = '';
        if ($ids) {
            $aWhere = array();
            foreach ($ids as $id) {
                $aWhere[] = "'" . addslashes($id) . "'";
            }

            $sWhere = ' && `id` IN (' . implode(',', $aWhere) . ')';
        }

        $results = $this->query("SELECT * FROM {$this->table} WHERE (`status` IS NULL OR `status` <> 'CONFIRMED' OR `status` <> 'FAILED')" . $sWhere)->fetchAll();
        
        $res = null;
        foreach ($results as $result) {
            
            $data = array();
            $data['Request']['ReceiptId'] = $result['id'];
            if ($data_ins = $this->UpdateNewCheckStatus($data)) {
                $res = $this->UpdateNewCheckInDB($result['id'], $data_ins);
            } else {
                $res = $this->UpdateFailedCheckInDB($result['id']);
            }
        }
        
        if ($ids && count($ids)==1){
            return $res;
        }
    }

    private function UpdateNewCheckStatus($data) {
        $res = $this->setAuthToken();
        if (!$res['status']) {
            return $res;
        }

        $ofd_api_url    = $this -> _config -> getOption('setting_api_url');
        $ofd_token      = $this -> _config -> getOption('setting_token');

        $options = $this->getHTTPOpt($data);
        $context = stream_context_create($options);
        set_error_handler(array($this, 'customErrorHandler'));
        try {
            $result = file_get_contents($ofd_api_url . "/status?AuthToken=" . $ofd_token, false, $context);
        } catch (Exception $e) {
            return $this->log_msg($e->getMessage());
        }
        restore_error_handler();
        $result = json_decode($result);
        if (isset($result->Status) && ($result->Status == 'Success')) {
            return $result->Data;
        } else if (isset($result->Status) && ($result->Status == 'Failed')) {
            return $this->log_msg($result->Error->Message);
        } else {
            return $this->log_msg('some error');
        }
    }

    private function UpdateNewCheckInDB($check_id, $data) {

        $aDataSave = array(
            "status"            => $data->StatusName,
            "status_message"    => $data->StatusMessage,
            "FN"                => $data->Device->FN,
            "RNM"               => $data->Device->RNM,
            "FDN"               => $data->Device->FDN,
            "FPD"               => $data->Device->FPD,
            "updated_at"        => gmdate('Y-m-d H:i:s'),
        );

        $aDataUpdate = array();
        foreach ($aDataSave as $key => $value) {
            $aDataUpdate[] = "`$key` = '" . addslashes($value) . "'";
        }

        if ($aDataUpdate) {
            $sql = "UPDATE {$this->table} SET " . implode(',', $aDataUpdate) . " WHERE `id` = '{$check_id}'";
            $this->query($sql);
        }
        
        return $this->log_msg("Статус для {$check_id} обновлен", true);
    }

    private function UpdateFailedCheckInDB($check_id) {
        return $this->log_msg("Ошибка обновления для {$check_id}");
    }

    public function setAuthToken() {
        if (!$this->checkSettings()) {
            return $this->log_msg('Для корректной работы заполните необходимы настройки');
        };

        if ($this->checkToken()) {
            return $this->log_msg('', true);
        }

        $ofd_auth_url  = $this -> _config -> getOption('setting_auth_url');
        $ofd_login     = $this -> _config -> getOption('setting_login');
        $ofd_pass      = $this -> _config -> getOption('setting_password');

        $data = array(
            "Login"     => $ofd_login,
            "Password"  => $ofd_pass,
        );

        $options = $this->getHTTPOpt($data);
        $context = stream_context_create($options);
        set_error_handler(array($this, 'customErrorHandler'));

        try {
            $result = file_get_contents($ofd_auth_url, false, $context);
        } catch (Exception $e) {
            return $this->log_msg($e->getMessage());
        }

        restore_error_handler();
        $result = json_decode($result);

        if (isset($result->Status) && ($result->Status == 'Success')) {
            
            //Обновляем токен
			$this -> updateConfig(array(
				'setting_token' => $result->Data->AuthToken,
				'setting_token_exp_date' => $result->Data->ExpirationDateUtc
			));
                    
            return $this->log_msg('', true);
        } else if (isset($result->Status) && ($result->Status == 'Failed')) {
            return $this->log_msg($result->Error->Message);
        } else {
            return $this->log_msg('some error');
        }
    }

    private function getErrorByCode($code_id, $data) {

        $msg = '';

        SWITCH ($code_id) {
            case 1019:
                $msg = "Идентификатор счета '{$data['Request']['InvoiceId']}' уже существует в ОФД";
                break;
        }

        return $msg;
    }

    public function sendDataToOFD($data) {

        $res = $this->setAuthToken();
        if (!$res['status']) {
            return $res;
        }

        $ofd_api_url    = $this -> _config -> getOption('setting_api_url');
        $ofd_token      = $this -> _config -> getOption('setting_token');

        $options = $this->getHTTPOpt($data);
        $context = stream_context_create($options);

        set_error_handler(array($this, 'customErrorHandler'));
        try {
            $result = file_get_contents($ofd_api_url . "/receipt?AuthToken=" . $ofd_token, false, $context);
        } catch (Exception $e) {
            return $this->log_msg($e->getMessage());
        }

        restore_error_handler();
        $result = json_decode($result);

        if (isset($result->Status) && ($result->Status == 'Success')) {
            return $this->log_msg('', true, array('check_id' => $result->Data->ReceiptId));
        } else if (isset($result->Status) && ($result->Status == 'Failed')) {
            if ($result->Error->Code) {
                if ($val = $this->getErrorByCode($result->Error->Code, $data)) {
                    return $this->log_msg($val);
                }
            }

            return $this->log_msg($result->Error->Message);
        } else {
            return $this->log_msg('some error');
        }
    }

    private function saveCheckInDB($check_id, $order_id, $data, $total) {
        try {
            $dataSave = array(
                'id'        => $check_id,
                'type'      => $data['Request']['Type'],
                'inn'       => $data['Request']['Inn'],
                'order_id'  => $order_id,
                'total'     => $total,
                'created_at'=> gmdate('Y-m-d H:i:s'),
            );
            
            $check_item_id = $this->insert($dataSave);
            
            return $this->log_msg('', true, array('check_item_id' => $check_item_id));
        } catch (Exception $e) {
            return $this->log_msg($e->getMessage());
        }
    }

    public function OFDcreate($order_id, $type = 'Income') {
        
        wa('shop');
        $order = new shopOrderModel();  	
        $order_data = $order->getOrder($order_id);
        
        if( ! isset($order_data) || empty($order_data)) {
            return $this->log_msg("Заказ #{$order_id} не существует");
        }

        //Тип чека
        $temptype = $this->getTextType($type);
        
        if ( ! $temptype){
            return $this->log_msg("Попытка сформировать неизвестный тип чека: {$temptype} - для заказа #{$order_id}");
        }
      

        if ($this->checkExists($order_id, $type)) {
            return $this->log_msg("Чек {$temptype} для заказа #{$order_id} уже оформлен");
        } else {
            //Формируем чек
            $data = $this->prepareData($order_data, $type);
            if (isset($data['status']) && !$data['status']) {
                return $data;
            }

            $ans = $this->sendDataToOFD($data);
            if ($ans['check_id']) {
                $save = $this->saveCheckInDB($ans['check_id'], $order_id, $data, $order_data['total']);
                if (!$save['check_item_id']) {
                    return $this->log_msg("Ошибка сохранения чека {$temptype} для заказа #{$order_id}");
                }

                return $this->log_msg('Чек ' . $temptype . ' для заказа #' . $order_id . ' успешно зарегистрирован.', true);
            }

            return $ans;
        }
    }
    
	public function OFDCreateAuto($order_id, $action){
		
		$setting_status = $this -> _config -> getOption('setting_status');
		
		if ($setting_status == $action){
			$this->setDebug(1);
			$this->OFDcreate($order_id);
			$this->setDebug(0);
		}
	}
	
    private function _filterToSqlWhere($filter = array()){
        
        $result = '';
        
        $filter = array_filter($filter);
        if ($filter){
            $aWhere = array();
            
            if (isset($filter['type'])){
                $aWhere[] = "type = '".addslashes($filter['type'])."'";
            }
            if (isset($filter['order_id'])){
                $aWhere[] = "order_id = '".addslashes($filter['order_id'])."'";
            }
            if (isset($filter['sdate'])){
                $aWhere[] = "created_at >= '".date('Y-m-d 00:00:00', strtotime(addslashes($filter['sdate'])))."'";
            }
            if (isset($filter['edate'])){
                $aWhere[] = "created_at <= '".date('Y-m-d 23:59:59', strtotime(addslashes($filter['edate'])))."'";
            }
            
            if ($aWhere){
                $result = ' WHERE ' . implode(' && ', $aWhere);
            }
        }
        
        return $result;
    }
    
	public function getListNds($val = ''){
		
		$html = '
		<option value="Vat0" '.($val == 'Vat0' ? ' selected="selected"' : '').'>без НДС</option>
		<option value="Vat10" '.($val == 'Vat10' ? ' selected="selected"' : '').'>10%</option>
		<option value="Vat18" '.($val == 'Vat18' ? ' selected="selected"' : '').'>18%</option>                           
		<option value="CalculatedVat10110" '.($val == 'CalculatedVat10110' ? ' selected="selected"' : '').'>10/110</option>                           
		<option value="CalculatedVat18118" '.($val == 'CalculatedVat18118' ? ' selected="selected"' : '').'>18/118</option>';
		
		return $html;
	}
	
    public function getCount($filter = array()) {
        
        $where = $this -> _filterToSqlWhere($filter);
        
        return $this->query("SELECT COUNT(*) FROM ".$this->table.$where)->fetchField();
    }

    public function getList($offset = 0, $countPage = 30, $filter = array()) {

        $where = $this -> _filterToSqlWhere($filter);
        
        $list = $this->query("SELECT * FROM ".$this->table.$where." ORDER BY `created_at` DESC LIMIT ".(int)$offset.", ".(int)$countPage)->fetchAll();
        
        if ($list){
            $ofd_inn            = $this -> _config -> getOption('setting_inn');
            $ofd_check_url      = $this -> _config -> getOption('setting_check_url');
            $ofd_check_url_demo = $this -> _config -> getOption('setting_check_url_demo');
            
            foreach ($list as &$item) {
                
                $item['update'] = FALSE;
                $item['type_name'] = $this -> getTextType($item['type']);
                $item['created_at'] = date('d.m.Y h:i:s', strtotime(($item['created_at'])));
                        
                if ('CONFIRMED' == $item['status']){
                    if ($ofd_check_url_demo){
                        $item['id_link'] = "'{$ofd_check_url_demo}".ltrim($item['FDN'],0)."/{$item['FPD']}'";
                    }else{
                        $inn = $item['inn'];
                        if ( ! $inn){
                            $inn = $ofd_inn;
                        }
                        
                        $item['id_link'] = "'{$ofd_check_url}{$inn}/{$item['RNM']}/{$item['FN']}/".ltrim($item['FDN'],0)."/{$item['FPD']}'";
                    }
                }else{
                    $item['update'] = TRUE;
                }

            }
        }

        return $list;
    }
}